unit uCreateImage;

interface

uses
  System.SysUtils, System.Classes, Vcl.Graphics, System.Types, VCLTee.Chart;

type
  TdmCreateChart = class(TDataModule)
  private
    { Private declarations }
    function CreateChart: TChart;
  public
    { Public declarations }

    function CreateImage(aRect: TRect): TGraphic;
  end;

var
  Use3DCanvas: boolean;

implementation

uses
  VCLTee.Series, Vcl.Controls, VCLTee.TeEngine, VCLTee.TeCanvas;

{%CLASSGROUP 'Vcl.Controls.TControl'}

{$R *.dfm}

{ TDataModule2 }

function TdmCreateChart.CreateChart: TChart;
begin
  result := TChart.Create(nil);

  //result.Canvas := TTeeCanvas3D.Create;  doesn't allow transparent series for batch gap chart

  result.Color := clWhite;
  result.BevelOuter := bvNone;
  result.LeftAxis.Grid.Color := clSilver;


  result.LeftAxis.AutomaticMinimum := False;
  result.LeftAxis.Minimum := 0;
  result.LeftAxis.MinorTicks.Visible := False;
  result.LeftAxis.TicksInner.Visible := False;
  result.LeftAxis.Ticks.Style := psDot;
  result.LeftAxis.TickLength := 2;
  result.LeftAxis.Axis.Width := 1;
  result.LeftAxis.Title.Font.Name := 'Arial Narrow';

  result.BottomAxis.Axis.Color := clSilver;
  result.BottomAxis.MinorTicks.Visible := false;
  result.BottomAxis.Ticks.Visible := false;
  result.BottomAxis.Grid.Visible := False;
  result.BottomAxis.Axis.Width := 1;

  result.Legend.Frame.Visible := False;
  result.Legend.Title.Font.Name := 'Arial Narrow';
  result.Legend.Title.Font.Style := [];
  result.Legend.Shadow.VertSize := 0;
  result.Legend.LegendStyle := lsSeries;
  result.Legend.Shadow.HorizSize := 0;
  result.Legend.Alignment := laRight;
  result.Legend.Transparent := True;

  result.LeftAxis.Axis.Color := clSilver;
  result.LeftAxis.AxisValuesFormat := '$,0';

  result.Walls.Visible := False;

  result.BackWall.Pen.Color := clSilver;

  result.View3D := False;
end;

procedure AssignChartToGraphic(aChart: TChart; const aRect: TRect; aGraphic: TGraphic; aTransparent: boolean = false);
var
  lHoldShadow: boolean;
  I: Integer;
  lBoolArray: array of boolean;
  lHoldBevel: TBevelCut;
  lHoldColor: TColor;
  lHoldGradientVisible: boolean;
  lMetafile: TMetafile;
  {$ifopt C+}
  lCanvasClassName: string;
  {$endif}
  lBmp: TBitmap;
  lCanvasClass: TCanvas3DClass;
  lHoldHover: Boolean;
begin
  {$ifopt C+}
  lCanvasClassName := aChart.Canvas.ClassName;
  {$endif}
  lCanvasClass := TCanvas3DClass(aChart.Canvas.ClassType);

  if Use3DCanvas and (lCanvasClass <> TTeeCanvas3D) then
    aChart.Canvas := TTeeCanvas3D.Create;

  lHoldGradientVisible := aChart.Gradient.Visible;
  lHoldColor := aChart.Color;
  lHoldBevel := aChart.BevelOuter;
  lHoldShadow := aChart.Shadow.Visible;
  lHoldHover := aChart.Hover.Visible;

  try
    SetLength(lBoolArray, aChart.SeriesCount);
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
      begin
        lBoolArray[I] := TPieSeries(aChart.Series[I]).Circled;
        TPieSeries(aChart.Series[I]).Circled := True;
      end
      else
        lBoolArray[I] := True;


    aChart.Gradient.Visible  := false;
    if not aTransparent then
      aChart.Color := clWhite
    else
      aChart.Color := clNone; //clWhite;
    aChart.BevelOuter := bvNone;
    aChart.BevelInner := bvNone;
    aChart.Shadow.Visible := False;
    aChart.Hover.Visible := false;

    if aGraphic is TMetafile then
    begin
      lMetafile := aChart.TeeCreateMetafile(false, aRect);
      try
        aGraphic.Assign(lMetafile);
      finally
        lMetafile.Free;
      end;
    end
    else
    begin
      lBmp := aChart.TeeCreateBitmap(clWhite, aRect);
      try
        aGraphic.Assign(lBmp);
      finally
        lBmp.Free;
      end;
    end;
  finally
    aChart.Hover.Visible := lHoldHover;
    aChart.Shadow.Visible := lHoldShadow;
    aChart.Gradient.Visible := lHoldGradientVisible;
    aChart.Color := lHoldColor;
    aChart.BevelOuter := lHoldBevel;
    for I := 0 to aChart.SeriesCount - 1 do    // Iterate
      if aChart.Series[I] is TPieSeries then
        TPieSeries(aChart.Series[I]).Circled := lBoolArray[I];
    if (lCanvasClass <> aChart.Canvas.ClassType) then
      aChart.Canvas := lCanvasClass.Create;
  end;

  {$ifopt C+}
  assert(SameText(lCanvasClassName, aChart.Canvas.ClassName),
    Format('Chart canvas should be %s but is %s', [lCanvasClassName, aChart.Canvas.ClassName]));
  {$endif}
end;

function TdmCreateChart.CreateImage(aRect: TRect): TGraphic;
var
  lChart: TChart;
  lSrs: TAreaSeries;

  function CreateChartSeries: TAreaSeries;
  begin
    result := TAreaSeries.Create(self);
    result.ParentChart := lChart;
    result.AreaLinesPen.Visible := false;
    result.Pen.Visible := false;
  end;

begin
  lChart := CreateChart;
  try
//  lChart.Parent := self;

    lChart.LeftAxis.Title.Font.Size := 10;
    lChart.BottomAxis.Title.Caption := 'Years in Retirement';
    lChart.BottomAxis.Title.Font.Size := 10;
    lChart.Legend.CustomPosition := True;

    lSrs := CreateChartSeries;
    lSrs.FillSampleValues(100);

    lChart.SetBounds(aRect.Left, aRect.Top, aRect.Right, aRect.Bottom);

    lChart.Draw;
    lChart.Draw;

    result := TBitmap.Create;
    result.Height := aRect.Height;
    result.Width := aRect.Width;
    AssignChartToGraphic(lChart, aRect, result);

  finally // wrap up
    lChart.FreeAllSeries;
    lChart.Free;
  end;    // try/finally
end;

end.
